%----------------------------------------------------------------------
function [probSame, labeledParsing] = Chunk_ChangeDetection(display1, display2, K, Sm)
    
    numMontes = 1000; % Num samples to take to settle on a region breakdown
    whichSamp =      find(display1~=0); % Which nodes to sample
    possibleLabels = 1:length(whichSamp); % Labels each node can take on
    
    % For each 'subject', run a separate chain, with random initial
    % starting position
    for chain=1:15 
        whichColors = display1(display1~=0);
        
        % Randomize initial starting position
        cur = cell(1,numMontes);
        cur{1} = display1;
        cur{1}(display1~=0) = 1:length(whichSamp);
        cur{1}(display1==0) = 0;

        % Run Gibbs sampler to create region
        sampList = RandSample(whichSamp, [1 numMontes]);
        for i=2:numMontes
            cur{i} = cur{i-1};
            
            % Choose which object to sample
            j = sampList(i);
            
            % Calculate smoothness of labels for each possible label this
            % item could have
            n1 = GibbsProb([Sm Sm], cur{i}, j, possibleLabels, display1);
            
            % Calculate likelihood of this item in each region based on
            % whether it matches the color of the region            
            n2 = RegionProb(display1, j, possibleLabels, whichColors);
            
            % Combine and sample
            n = (n1.*n2);
            cur{i}(j) = possibleLabels(find(rand<cumsum(n./sum(n)),1));
        end

        % Save this parsing
        labeledParsing{chain} = cur{end};

        % Create a list of which indices are in which region
        regionList = unique(cur{end}(cur{end}~=0));
        clear regions;
        for i=1:length(regionList)
            regions{i} = find(cur{end}==regionList(i));
        end
        
        % ... and calculate the percent correct we'd expect based on this
        % segmentation        
        isSame(chain) = CalcProbCorrect(display1, display2, regions, K);
    end
    probSame = mean(isSame);
end

%----------------------------------------------------------------------
function p = RegionProb(display1, j, possibleLabels, whichPref)
    % Probability of item being in region is strictly dependent on whether
    % its' color matches that region
    for i=1:length(possibleLabels)
        if whichPref(i) == display1(j)
            likeCur(i) = 1;
        else
            likeCur(i) = 0;
        end
    end
    p = likeCur ./ sum(likeCur);
end


%----------------------------------------------------------------------
function p = GibbsProb(params, labels, j, possibleLabels, display1)
%----------------------------------------------------------------------
    persistent hPairs; persistent vPairs;
    if isempty(hPairs)
        % For speed: Cache the item indexes that are adjacent
        % horizontally and vertically so we don't have to recalculate them
        % everytime
        nn = (1:numel(labels))';
        nnD=nn+1;	nnR=nn+size(labels,1);
        hPairs = [nn nnR];
        hPairs = hPairs(nnR>=1&nnR<=numel(labels),:);
        vPairs = [nn nnD];
        cols = ceil(vPairs./size(labels,1));
        vPairs = vPairs(nnD>=1&nnD<=numel(labels)&(cols(:,1)==cols(:,2)),:);
    end
    
    % Remove the pairs that contain blank spots from the display
    %
    % Note: This isn't done in the persistent part of the function because
    % persistent things in Matlab stick around even outside the runs of the
    % Chunk_ChangeDetection() call.
    %
    w = find(display1);	cut = [];
    for i=1:length(hPairs)
        if ~(any(w==hPairs(i,1)) && any(w==hPairs(i,2))), 
            cut = [cut i]; 
        end
    end
    hPairsNew = hPairs;
    hPairsNew(cut,:) = [];	cut = [];
    for i=1:length(vPairs)
        if ~(any(w==vPairs(i,1)) && any(w==vPairs(i,2)))
            cut = [cut i];
        end
    end
    vPairsNew = vPairs;
    vPairsNew(cut,:) = [];
        
    % Calculate the smoothness of the set of labels for each possible label
    % that item j could take on    
    for i=1:length(possibleLabels)
        labels(j) = possibleLabels(i);
        valsH = labels(hPairsNew);
        valsV = labels(vPairsNew);
        valsH(valsH(:,1)==0 | valsH(:,2)==0, :) = [];
        valsV(valsV(:,1)==0 | valsV(:,2)==0, :) = [];
        hSum = sum(valsH(:,1)==valsH(:,2))*1 + sum(valsH(:,1)~=valsH(:,2))*-1;
        vSum = sum(valsV(:,1)==valsV(:,2))*1 + sum(valsV(:,1)~=valsV(:,2))*-1;
        likeCur(i) = exp(params(1)*hSum + params(2)*vSum);
    end
    p = likeCur ./ sum(likeCur);
end

%----------------------------------------------------------------------
function probSame = CalcProbCorrect(display1, display2, regions, K)
    % Next count up how many objects are in each region
    for i=1:length(regions)
        regions{i} = regions{i}';
        chanceThisRegion(i) = length(regions{i})/numel(display1(display1~=0));
    end

    % If there are fewer than K objects, remember all of them
    if K>length(regions)
        K = length(regions);
    end

    % Which regions we choose to encode into memory
    choices = combs(1:length(regions), K);
    for i=1:size(choices,1)

        % P(ThisSet) Choosing the K regions
        which = zeros(1, length(regions));
        which(choices(i,:)) = 1;
        encodeThisSet(i) = mnpdf(which, chanceThisRegion);

        % Which objects are in the K regions
        objectsThisSet{i} = [regions{choices(i,:)}];
    end

    % Normalize
    encodeThisSet   = encodeThisSet  ./ sum(encodeThisSet);

    % For each set of K we could encode
    isSame = zeros(1,length(encodeThisSet));
    for i=1:length(encodeThisSet)
        % If one of those objects changed, we notice
        if any(display1(objectsThisSet{i}) ~= display2(objectsThisSet{i}))
            isSame(i) = 0;
            continue;
        end

        % If not, we use our sampling strategy to infer how likely to be the
        % same the display is (e.g., if we remember more objects it is more
        % likely to be the same).
        numObjectsEncoded = length(objectsThisSet{i});
        isSame(i) = 0.5 + 0.5*numObjectsEncoded/numel(display1);
    end

    probSame = sum(isSame .* encodeThisSet);
end

%----------------------------------------------------------------------
function P = combs(v,m)
% Helper function
% COMBS  All possible combinations.
%   COMBS(1:N,M) or COMBS(V,M) where V is a row vector of length N,
%   creates a matrix with N!/((N-M)! M!) rows and M columns containing
%   all possible combinations of N elements taken M at a time.
%
    if nargin~=2
        error('MATLAB:nchoosek:WrongInputNum', 'Requires 2 input arguments.');
    end
    v = v(:).'; % Make sure v is a row vector.
    n = length(v);
    if n == m
        P = v;
    elseif m == 1
        P = v.';
    else
        P = [];
        if m < n && m > 1
            for k = 1:n-m+1
                Q = combs(v(k+1:n),m-1);
                P = [P; [v(ones(size(Q,1),1),k) Q]];
            end
        end
    end
end
